/*
 * ******************************************************************************************
 *
 * File		: apb_rtc.h
 * Author	: GOWIN
 * Function	: Definition of APB RTC
 *
 * ******************************************************************************************
 */

#ifndef __APB_RTC_H__
#define __APB_RTC_H__


#ifdef __cplusplus
extern "C" {
#endif


// Includes ---------------------------------------------------------------------------------
#include <stddef.h>


/*****************************************************************************
 * Device Specific Peripheral Registers structures
 ****************************************************************************/
#define __RO          volatile const  /* 'read  only' permissions     */
#define __WO          volatile        /* 'write only' permissions     */
#define __RW          volatile        /* 'read/write' permissions     */


/*****************************************************************************
 * APB RTC (Real Time Clock)
 ****************************************************************************/
typedef struct
{
	__RO unsigned int IDREV;                /* 0x00 ID and Revision Register */
	     unsigned int RESERVED[3];          /* 0x04~0x0C Reserved            */
	__RW unsigned int CNTR;                 /* 0x10 Counter Register         */
	__RW unsigned int ALARM;                /* 0x14 Alarm Register           */
	__RW unsigned int CTRL;                 /* 0x18 Control Register         */
	__RW unsigned int STATUS;               /* 0x1C Status Register          */
	__RW unsigned int TRIM;					/* 0x20 Digit Trimming Register  */
} APB_RTC_RegDef;


/* General return codes */
#define APB_RTC_DRIVER_OK                 0 // Operation succeeded
#define APB_RTC_DRIVER_ERROR             -1 // Unspecified error
#define APB_RTC_DRIVER_ERROR_BUSY        -2 // Driver is busy
#define APB_RTC_DRIVER_ERROR_TIMEOUT     -3 // Timeout occurred
#define APB_RTC_DRIVER_ERROR_UNSUPPORTED -4 // Operation not supported
#define APB_RTC_DRIVER_ERROR_PARAMETER   -5 // Parameter error
#define APB_RTC_DRIVER_ERROR_SPECIFIC    -6 // Start of driver specific errors

// APB RTC status
typedef struct _APB_RTC_STATUS
{
    unsigned int Status;
} APB_RTC_STATUS;

// APB RTC control ID
typedef enum _APB_RTC_CTRL_ID
{
	APB_RTC_CTRL_NONE,
	APB_RTC_CTRL_EN,
	APB_RTC_CTRL_ALARM_WAKEUP,
	APB_RTC_CTRL_ALARM_INT,
	APB_RTC_CTRL_DAY_INT,
	APB_RTC_CTRL_HOUR_INT,
	APB_RTC_CTRL_MIN_INT,
	APB_RTC_CTRL_SEC_INT,
	APB_RTC_CTRL_HSEC_INT,
} APB_RTC_CTRL_ID;

// APB RTC event ID
#define APB_RTC_EVENT_ALARM_INT     (1UL << 0)
#define APB_RTC_EVENT_DAY_INT       (1UL << 1)
#define APB_RTC_EVENT_HOUR_INT      (1UL << 2)
#define APB_RTC_EVENT_MIN_INT       (1UL << 3)
#define APB_RTC_EVENT_SEC_INT       (1UL << 4)
#define APB_RTC_EVENT_HSEC_INT      (1UL << 5)

// Callback function
typedef void (*APB_RTC_SignalEvent_t) (unsigned int event);

// APB RTC driver time format.
typedef struct _APB_RTC_TIME
{
    unsigned int day;
    unsigned int hour;
    unsigned int min;
    unsigned int sec;
} APB_RTC_TIME;

// APB RTC driver date format.
typedef struct _APB_RTC_ALARM
{
	unsigned int hour;
	unsigned int min;
	unsigned int sec;
} APB_RTC_ALARM;

/**
\brief General power states
*/
typedef enum _APB_RTC_POWER_STATE
{
	APB_RTC_POWER_OFF,                      // Power off: no operation possible
	APB_RTC_POWER_LOW,                      // Low power mode: retain state, detect and signal wake-up events
	APB_RTC_POWER_FULL                      // Power on: full operation at maximum performance
} APB_RTC_POWER_STATE;


#ifndef BIT
#define BIT(n)                      ((unsigned int) 1 << (n))
#define BITS2(m,n)                  (BIT(m) | BIT(n) )

/* Bits range: for example BITS(16,23) = 0xFF0000
 *   ==>  (BIT(m)-1)   = 0x0000FFFF     ~(BIT(m)-1)   => 0xFFFF0000
 *   ==>  (BIT(n+1)-1) = 0x00FFFFFF
 */
#define BITS(m,n)                   (~(BIT(m)-1) & ((BIT(n) - 1) | BIT(n)))
#endif /* BIT */

#define APB_RTC_HALF_SECOND_SUPPORT   	(0)
#define APB_RTC_DAY_BITS              	(9)        	   // for counting up to 2^9 = 512 days

// Counter register bit define
// Day passed after APB RTC enabled, BITS(17,21), counting up to 2^9 = 512 days
#define CNTR_DAY                    BITS(17,(17 + (APB_RTC_DAY_BITS - 1)))
#define CNTR_HOUR                   BITS(12,16)    // Hour field of current time; range: 0 ~ 23
#define CNTR_MIN                    BITS(6,11)     // Minute field of current time; range: 0 ~ 59
#define CNTR_SEC                    BITS(0,5)      // Second field of current time; range: 0 ~ 59

// Alarm register bit define
#define ALARM_HOUR                  BITS(12,16)    // Hour field of current time; range: 0 ~ 23
#define ALARM_MIN                   BITS(6,11)     // Minute field of current time; range: 0 ~ 59
#define ALARM_SEC                   BITS(0,5)      // Second field of current time; range: 0 ~ 59

// Control register bit define
#define CTRL_RTC_EN                 BIT(0)         // Enable APB RTC
#define CTRL_ALARM_WAKEUP           BIT(1)         // Enable alarm wake up signal
#define CTRL_ALARM_INT              BIT(2)         // Enable alarm interrupt
#define CTRL_DAY                    BIT(3)         // Enable day interrupt
#define CTRL_HOUR                   BIT(4)         // Enable hour interrupt
#define CTRL_MIN                    BIT(5)         // Enable minute interrupt
#define CTRL_SEC                    BIT(6)         // Enable second interrupt
#define CTRL_HSEC                   BIT(7)         // Enable half-second interrupt

// Status register bit define
#define STATUS_ALARM_INT            BIT(2)         // Alarm interrupt status
#define STATUS_DAY                  BIT(3)         // Day interrupt status
#define STATUS_HOUR                 BIT(4)         // Hour interrupt status
#define STATUS_MIN                  BIT(5)         // Minute interrupt status
#define STATUS_SEC                  BIT(6)         // Second interrupt status
#define STATUS_HSEC                 BIT(7)         // Half-second interrupt status
#define STATUS_WRITEDONE            BIT(16)        // The synchronization progress of APB RTC register updates

// Digit trimming register bit define
#define TRIM_SEC_TRIM				BITS(0,4)		// Digital trimming value for the rest of seconds
#define TRIM_SEC_SIGN				BIT(7)			// Sign bit for the trimming value for the rest of seconds
#define TRIM_MIN_TRIM				BITS(8:12)		// Digital trimming value for the second on the minute boundary
#define TRIM_MIN_SIGN				BIT(15)			// Sign bit for the trimming value for the second on the minute boundary
#define TRIM_HOUR_TRIM				BITS(16,20)		// Digital trimming value for the second on the hour boundary
#define TRIM_HOUR_SIGN				BIT(23)			// Sign bit for the trimming value for the second on the hour boundary
#define TRIM_DAY_TRIM				BITS(24,28)		// Digital trimming value for the second on the day boundary
#define TRIM_DAY_SIGN				BIT(31)			// Sign bit for the trimming value for the second on the day boundary

// APB RTC information
typedef struct _APB_RTC_INFO
{
	APB_RTC_SignalEvent_t   Sig_Evt_CB;
	APB_RTC_TIME            Set_Rtc_Time;
	APB_RTC_ALARM           Set_Rtc_Alarm;
	APB_RTC_POWER_STATE     Pwr_State;
} APB_RTC_INFO;

// APB RTC resources definitions
typedef struct
{
	APB_RTC_RegDef		*reg;				// Point to APB RTC peripheral
	unsigned int		irq_period_num;		// APB RTC period IRQ number
	unsigned int		irq_alarm_num;		// APB RTC alarm IRQ number
	APB_RTC_INFO		*info;				// APB RTC run-time information
} const APB_RTC_RESOURCES;


#ifdef __cplusplus
}
#endif


#endif	// __APB_RTC_H__
