/*
 * ******************************************************************************************
 *
 * File		: apb_rtc_driver.c
 * Author	: GOWIN
 * Function	: Driver definition of APB RTC
 *
 * ******************************************************************************************
 */


// Includes ---------------------------------------------------------------------------------
#include "apb_rtc_driver.h"
#include "apb_rtc_hal.h"


// Definitions ------------------------------------------------------------------------------

// Initializes APB RTC
int apb_rtc_initialize(APB_RTC_SignalEvent_t cb_event, APB_RTC_RESOURCES* rtc)
{
	unsigned int Tmp_C = 0, Tmp_S;

	Tmp_C = rtc->reg->CTRL;
	Tmp_S = rtc->reg->STATUS;

	// Disable all interrupt and clear interrupt status
	rtc->reg->CTRL = (Tmp_C & (~ BITS(0,7)));
	rtc->reg->STATUS = (Tmp_S | BITS(2,7));

	// Enable APB RTC period interrupt
	hal_enable_interrupt(rtc->irq_period_num);

	// Enable APB RTC alarm interrupt
	hal_enable_interrupt(rtc->irq_alarm_num);

	rtc->info->Sig_Evt_CB = cb_event;
	rtc->info->Pwr_State = APB_RTC_POWER_FULL;

	return APB_RTC_DRIVER_OK;
}

// Uninitialized
int apb_rtc_uninitialize(APB_RTC_RESOURCES* rtc)
{
	unsigned int Tmp_C = 0, Tmp_S;

	Tmp_C = rtc->reg->CTRL;
	Tmp_S = rtc->reg->STATUS;

	// Disable all interrupt and clear interrupt status
	rtc->reg->CTRL = (Tmp_C & (~ BITS(0,7)));
	rtc->reg->STATUS = (Tmp_S | BITS(2,7));

	// Disable APB RTC period interrupt
	hal_disable_interrupt(rtc->irq_period_num);

	// Disable APB RTC alarm interrupt
	hal_disable_interrupt(rtc->irq_alarm_num);

	rtc->info->Sig_Evt_CB = NULL;
	rtc->info->Pwr_State = APB_RTC_POWER_OFF;

	return APB_RTC_DRIVER_OK;
}

// Power control
int apb_rtc_power_control(APB_RTC_POWER_STATE state, APB_RTC_RESOURCES* rtc)
{
	rtc->info->Pwr_State = state;

	switch (state)
	{
	case APB_RTC_POWER_OFF:
		break;
	case APB_RTC_POWER_LOW:
		break;
	case APB_RTC_POWER_FULL:
		break;
	default:
		return APB_RTC_DRIVER_ERROR_UNSUPPORTED;
	}

	return APB_RTC_DRIVER_OK;
}

// Set time
int apb_rtc_set_time(APB_RTC_TIME* stime, APB_RTC_RESOURCES* rtc)
{
	unsigned int Tmp_C = 0, Tmp_S;

	Tmp_S = rtc->reg->STATUS;

	if (rtc->info->Pwr_State != APB_RTC_POWER_FULL)
	{
		return APB_RTC_DRIVER_ERROR_UNSUPPORTED;
	}

	do
	{
		Tmp_S = rtc->reg->STATUS;
	// Wait until the WriteDone field of the Status Register equals 1
	} while (!(Tmp_S & STATUS_WRITEDONE));

	if ((stime->day >= (0x1 << APB_RTC_DAY_BITS)) ||
		(stime->hour >= 24) ||
		(stime->min >= 60)  ||
		(stime->sec >= 60))
	{
		return APB_RTC_DRIVER_ERROR_PARAMETER;
	}

	// Keep the current set time for reference
	rtc->info->Set_Rtc_Time.day = stime->day;
	rtc->info->Set_Rtc_Time.hour = stime->hour;
	rtc->info->Set_Rtc_Time.min = stime->min;
	rtc->info->Set_Rtc_Time.sec = stime->sec;

	Tmp_C = (stime->day << 17) | (stime->hour << 12) | (stime->min << 6) | (stime->sec << 0);

	// Set to APB RTC HW IP
	rtc->reg->CNTR = Tmp_C;

	return APB_RTC_DRIVER_OK;
}

// Get time
int apb_rtc_get_time(APB_RTC_TIME* stime, APB_RTC_RESOURCES* rtc)
{
	unsigned int Tmp = 0;

	if (rtc->info->Pwr_State != APB_RTC_POWER_FULL)
	{
		return APB_RTC_DRIVER_ERROR_UNSUPPORTED;
	}

	// Get from APB RTC HW IP
	Tmp = rtc->reg->CNTR;

	// Return the current time queried from HW
	stime->day = ((Tmp & CNTR_DAY) >> 17);
	stime->hour = ((Tmp & CNTR_HOUR) >> 12);
	stime->min = ((Tmp & CNTR_MIN) >> 6);
	stime->sec = ((Tmp & CNTR_SEC) >> 0);

	if ((stime->day >= (0x1 << APB_RTC_DAY_BITS)) ||
		(stime->hour >= 24) ||
		(stime->min >= 60)  ||
		(stime->sec >= 60))
	{
		return APB_RTC_DRIVER_ERROR_PARAMETER;
	}

	return APB_RTC_DRIVER_OK;
}

// Set alarm
int apb_rtc_set_alarm(APB_RTC_ALARM* salarm, APB_RTC_RESOURCES* rtc)
{
	unsigned int Tmp_C = 0, Tmp_S;

	Tmp_S = rtc->reg->STATUS;

	if (rtc->info->Pwr_State != APB_RTC_POWER_FULL)
	{
		return APB_RTC_DRIVER_ERROR_UNSUPPORTED;
	}

	do
	{
		Tmp_S = rtc->reg->STATUS;
	// Wait until the WriteDone field of the Status Register equals 1
	} while(!(Tmp_S & STATUS_WRITEDONE));

	if ((salarm->hour >= 24) ||
		(salarm->min >= 60)  ||
		(salarm->sec >= 60))
	{
		return APB_RTC_DRIVER_ERROR_PARAMETER;
	}

	// Keep the current alarm for reference
	rtc->info->Set_Rtc_Alarm.hour = salarm->hour;
	rtc->info->Set_Rtc_Alarm.min = salarm->min;
	rtc->info->Set_Rtc_Alarm.sec = salarm->sec;

	Tmp_C = (salarm->hour << 12) | (salarm->min << 6) | (salarm->sec << 0);

	// Set to APB RTC HW IP
	rtc->reg->ALARM = Tmp_C;

	return APB_RTC_DRIVER_OK;
}

// Get alarm
int apb_rtc_get_alarm(APB_RTC_ALARM* salarm, APB_RTC_RESOURCES* rtc)
{
	unsigned int Tmp = 0;

	if (rtc->info->Pwr_State != APB_RTC_POWER_FULL)
	{
		return APB_RTC_DRIVER_ERROR_UNSUPPORTED;
	}

	// Get from APB RTC HW IP
	Tmp = rtc->reg->ALARM;

	// Return the current alarm queried from HW
	salarm->hour = ((Tmp & ALARM_HOUR) >> 12);
	salarm->min = ((Tmp & ALARM_MIN) >> 6);
	salarm->sec = ((Tmp & ALARM_SEC) >> 0);

	if ((salarm->hour >= 24) ||
		(salarm->min >= 60)  ||
		(salarm->sec >= 60))
	{
		return APB_RTC_DRIVER_ERROR_PARAMETER;
	}

	return APB_RTC_DRIVER_OK;
}

// Control
int apb_rtc_control(unsigned int control, unsigned int arg, APB_RTC_RESOURCES* rtc)
{
	APB_RTC_CTRL_ID eCtrlId = (APB_RTC_CTRL_ID)control;
	unsigned int Tmp_C = 0, Tmp_S;

	Tmp_C = rtc->reg->CTRL;
	Tmp_S = rtc->reg->STATUS;

	if (rtc->info->Pwr_State != APB_RTC_POWER_FULL)
	{
		return APB_RTC_DRIVER_ERROR_UNSUPPORTED;
	}

	do
	{
		Tmp_S = DEV_RTC->STATUS;
	// Wait until the WriteDone field of the Status Register equals 1
	} while (!(Tmp_S & STATUS_WRITEDONE));

	switch (eCtrlId)
	{
	case APB_RTC_CTRL_NONE:
		break;
	case APB_RTC_CTRL_EN:
		if (arg == 1)
		{
			// APB RTC ON
			rtc->reg->CTRL = (Tmp_C | CTRL_RTC_EN);
		}
		else if (arg == 0)
		{
			// APB RTC OFF
			rtc->reg->CTRL = (Tmp_C & ~CTRL_RTC_EN);
		}

		break;

	case APB_RTC_CTRL_ALARM_WAKEUP:
		if (arg == 1)
		{
			// INT ON
			rtc->reg->CTRL = (Tmp_C | CTRL_ALARM_WAKEUP);
		}
		else if (arg == 0)
		{
			// INT OFF
			rtc->reg->CTRL = (Tmp_C & ~CTRL_ALARM_WAKEUP);
		}

		break;

	case APB_RTC_CTRL_ALARM_INT:
		// Write 1 clear
		rtc->reg->STATUS = (Tmp_S | STATUS_ALARM_INT);

		if (arg == 1)
		{
			// INT ON
			rtc->reg->CTRL = (Tmp_C | CTRL_ALARM_INT);
		}
		else if (arg == 0)
		{
			// INT OFF
			rtc->reg->CTRL = (Tmp_C & ~CTRL_ALARM_INT);
		}

		break;

	case APB_RTC_CTRL_DAY_INT:
		// Write 1 clear
		DEV_RTC->STATUS = (Tmp_S | STATUS_DAY);

		if (arg == 1)
		{
			// INT ON
			rtc->reg->CTRL = (Tmp_C | CTRL_DAY);
		}
		else if (arg == 0)
		{
			// INT OFF
			rtc->reg->CTRL = (Tmp_C & ~CTRL_DAY);
		}

		break;

	case APB_RTC_CTRL_HOUR_INT:
		// Write 1 clear
		rtc->reg->STATUS = (Tmp_S | STATUS_HOUR);

		if (arg == 1)
		{
			// INT ON
			rtc->reg->CTRL = (Tmp_C | CTRL_HOUR);
		}
		else if (arg == 0)
		{
			// INT OFF
			rtc->reg->CTRL = (Tmp_C & ~CTRL_HOUR);
		}

		break;

	case APB_RTC_CTRL_MIN_INT:
		// Write 1 clear
		rtc->reg->STATUS = (Tmp_S | STATUS_MIN);

		if (arg == 1)
		{
			// INT ON
			rtc->reg->CTRL = (Tmp_C | CTRL_MIN);
		}
		else if (arg == 0)
		{
			// INT OFF
			rtc->reg->CTRL = (Tmp_C & ~CTRL_MIN);
		}

		break;

	case APB_RTC_CTRL_SEC_INT:
		// Write 1 clear
		rtc->reg->STATUS = (Tmp_S | STATUS_SEC);

		if (arg == 1)
		{
			// INT ON
			rtc->reg->CTRL = (Tmp_C | CTRL_SEC);
		}
		else if (arg == 0)
		{
			// INT OFF
			rtc->reg->CTRL = (Tmp_C & ~CTRL_SEC);
		}

		break;

	case APB_RTC_CTRL_HSEC_INT:
		// Write 1 clear
		rtc->reg->STATUS = (Tmp_S | STATUS_HSEC);

		if (arg == 1)
		{
			// INT ON
			rtc->reg->CTRL = (Tmp_C | CTRL_HSEC);
		}
		else if (arg == 0)
		{
			// INT OFF
			rtc->reg->CTRL = (Tmp_C & ~CTRL_HSEC);
		}

		break;

	default:
		return APB_RTC_DRIVER_ERROR_UNSUPPORTED;
	}

	return APB_RTC_DRIVER_OK;
}

// Get status
APB_RTC_STATUS apb_rtc_get_status(APB_RTC_RESOURCES* rtc)
{
	APB_RTC_STATUS  rRtc_Status;

	rRtc_Status.Status = rtc->reg->STATUS;

	return rRtc_Status;
}

// Event
void apb_rtc_signal_event(unsigned int event, APB_RTC_RESOURCES* rtc)
{
	// Invoke callback function
	if (rtc->info->Sig_Evt_CB)
	{
		rtc->info->Sig_Evt_CB(event);
	}
}

// APB RTC period interrupt handler
void apb_rtc_period_irq_handler(APB_RTC_RESOURCES* rtc)
{
	unsigned int Evt_Id = 0;
	unsigned int Tmp_S, Tmp_W;

	// Only check day/hour/minute/second/half-second interrupt, not include alarm interrupt
	Tmp_W = Tmp_S = (rtc->reg->STATUS & BITS(3,7));

	// Write 1 clear all interrupt status
	rtc->reg->STATUS = Tmp_W;

	do
	{
		// Day interrupt
		if (Tmp_S & STATUS_DAY)
		{
			Evt_Id |= APB_RTC_EVENT_DAY_INT;
		}
		// Hour interrupt
		if (Tmp_S & STATUS_HOUR)
		{
			Evt_Id |= APB_RTC_EVENT_HOUR_INT;
		}
		// Minute interrupt
		if (Tmp_S & STATUS_MIN)
		{
			Evt_Id |= APB_RTC_EVENT_MIN_INT;
		}
		// Second interrupt
		if (Tmp_S & STATUS_SEC)
		{
			Evt_Id |= APB_RTC_EVENT_SEC_INT;
		}
		// Half second interrupt
		if (Tmp_S & STATUS_HSEC)
		{
			Evt_Id |= APB_RTC_EVENT_HSEC_INT;
		}

		apb_rtc_signal_event(Evt_Id, rtc);
	} while (0);
}

// APB RTC alarm interrupt handler
void apb_rtc_alarm_irq_handler(APB_RTC_RESOURCES* rtc)
{
	unsigned int Evt_Id = 0;
	unsigned int Tmp_S, Tmp_W;

	// Only check alarm interrupt
	Tmp_W = Tmp_S = (rtc->reg->STATUS & BIT(2));

	// Write 1 clear all interrupt status
	rtc->reg->STATUS = Tmp_W;

	do
	{
		if (Tmp_S & STATUS_ALARM_INT)
		{
			Evt_Id |= APB_RTC_EVENT_ALARM_INT;
		}

		apb_rtc_signal_event(Evt_Id, rtc);
	} while (0);
}
